-- fire
-- Mod for Minetest 0.4
-- Licensed under the GNU GPLv3

-- Replace a burnable node by fire node
-- Spread the fire nodes on all the birnable nodes in radius by ABM
-- Destroy the fire nodes by ABM

-- TODO:
-- lighter
-- bomb

local FIRE_SPREAD_DELAY = 7
local FIRE_DISAPPEAR_DELAY = 8
local SMOKE_DISAPPEAR_DELAY = 8
local SMOKE_COLLISION_BOX = 1

--[[
function Set (list)
    local set = {}
    for _, l in ipairs(list) do set[l] = true end
    return set
end

local flammable = Set {
    "default:tree",
    "default:wood",
    "default:jungletree",
    "default:leaves"
}
]]

-- Deprecated: Smoke
minetest.register_node("fire:smoke", {
    drawtype = "plantlike",
    tile_images = { "fire_smoke.png" },
    inventory_image = "fire_smoke.png",
    paramtype = "light",
    is_ground_content = true,
    walkable = false,
    pointable = false,
    sunlight_propagates = true,
    alpha = 150,
    post_effect_color = {a = 80, r = 0, g = 0, b = 0}
})

-- Fire
minetest.register_node("fire:fire", {
    drawtype = "plantlike",
    tile_images = { "fire_fire.png" },
    inventory_image = "fire_fire.png",
    paramtype = "light",
    is_ground_content = true,
    walkable = false,
    pointable = false,
    sunlight_propagates = true,
    damage_per_second = 1,
    alpha = 150,
    light_source = 13,
    post_effect_color = {a = 100, r = 255, g = 0, b = 0}
})

-- Match
minetest.register_craftitem("fire:match" , {
    image = "fire_match.png",
    stack_max = 99,

    on_use = function(item, player, pointed_thing)
        if pointed_thing.type == "node" then
            print("[fire] Match used on (" .. pointed_thing.under.x .. ',' .. pointed_thing.under.y .. ',' .. pointed_thing.under.z .. ')')
            if(math.random(1, 6) ~= 1) then
                return burn(pointed_thing.under)
            else
                return true
            end
        end
        return false
    end
})

-- Match box
minetest.register_craftitem("fire:match_box", {
    image = "fire_match_box.png",
    stack_max = 99
})

-- Crafting match
minetest.register_craft({
    output = 'craft "fire:match" 10',
    recipe = {
        {'craft "default:coal_lump"' },
        {'craft "default:stick"' },
        {'craft "default:stick"' },
    }
})

-- Crafting match box
minetest.register_craft({
    output = 'craft "fire:match_box" 1',
    recipe = {
        { 'craft "fire:match"', 'craft "fire:match"', 'craft "fire:match' },
        { 'craft "fire:match"', 'craft "fire:match"', 'craft "fire:match' },
        { 'craft "fire:match"', 'craft "fire:match"', 'craft "fire:match' }
    }
})
minetest.register_craft({
    output = 'craft "fire:match" 9',
    recipe = {
        { 'craft "fire:match_box"'}
    }
})

-- Removing fire
minetest.register_abm({
    nodenames = { "fire:fire" },
    interval = FIRE_DISAPPEAR_INTERVAL,
    chance = 4,

    action = function(pos, node, _, __)
        minetest.env:remove_node(pos)
        create_smoke(pos)
    end
})

-- Deprecated: Removing smoke
minetest.register_abm({
    nodenames = { "fire:smoke" },
    interval = FIRE_DISAPPEAR_INTERVAL,
    chance = 4,

    action = function(pos, node, _, __)
        minetest.env:remove_node(pos)
    end
})

-- Spreading fire
minetest.register_abm({
    nodenames = { "fire:fire" },
    interval = FIRE_DISAPPEAR_INTERVAL,
    chance = 4,

    action = function(pos, node, _, __)
        for i = -1, 1 do
        for j = -1, 1 do
        for k = -1, 1 do
            if (math.abs(i) + math.abs(j) + math.abs(k) == 1) then
                local new_pos = {
                    x = pos.x + i,
                    y = pos.y + j,
                    z = pos.z + k
                }
                burn(new_pos)
            end
        end
        end
        end
    end
})

-- Burning a node. Can be used in other mods.
function burn(pos)
    local node = minetest.env:get_node(pos)
    
    -- Checking for water nearby
    for i = -1, 1 do
    for j = -1, 1 do
    for k = -1, 1 do
        local water_pos = {
            x = pos.x + i,
            y = pos.y + j,
            z = pos.z + k
        }
        local water_node = minetest.env:get_node(water_pos)
        if water_node.name == "default:water_source"
        or water_node.name == "default:water_flowing" then
            return false
        end
    end
    end
    end

    -- Checking for flammability
    if minetest.registered_nodes[node.name].furnace_burntime == nil then
        return false
    end
    if minetest.registered_nodes[node.name].furnace_burntime > -1 then
        minetest.env:remove_node(pos)
        minetest.env:add_node(pos, {name = "fire:fire"})
        create_smoke(pos)
        return true
    else
        return false
    end
end

-- Smoke entity
minetest.register_entity( "fire:smoke_ent", {
    physical = false,
    textures = { "fire_smoke.png" },
    timer = 0,

    on_step = function(self, dtime)
        self.timer = self.timer + dtime
        if(self.timer >= math.random(3, 10))
        or minetest.env:get_node(self.object:getpos()).name ~= "air" then
            self.object:remove()
        end
    end
})

-- Creating smoke
function create_smoke(pos)
    local pos_above = {
        x = pos.x,
        y = pos.y + 1,
        z = pos.z
    }
    if minetest.env:get_node(pos_above).name ~= "air" then
        return
    end
    local smoke = minetest.env:add_entity({
        x = pos.x,
        y = pos.y,
        z = pos.z,
    }, "fire:smoke_ent")
    smoke:setvelocity({
        x = 0,
        y = math.random(1, 3),
        z = 0
    })
end
